--- AWS  usage via script

-- @file    aws.lua
-- @author  Yuriy Zveryanskyy
-- @created 2021/12/07

local aws = {}

local script = "/usr/bin/aws"

--- Check that we can operate with aws
function aws.enabled()
    return require"core.fsutils".is_file(script)
end

--- Send file to S3 storage
-- @param datatype    Type of the data
-- @param file        Path to the file
-- @param extension   Extension for save
function aws.sendToS3(datatype, file, extension)
    local bucket = "pbx-cloud-storage"
    local types = {
        ["record"] = "callrecordings",
        ["voicemail"] = "voicemail",
        ["fax"] = "faxes"
    }

    local pbx_serial = require"core.pbxutils".pbxSerial()
    if not pbx_serial then
        return nil, "pbx serial is not found"
    end

    local dsttype = types[datatype]
    if not dsttype then
        return nil, "invalid data type"
    end

    local url = string.format("s3://%s/%s/%s", bucket, pbx_serial, dsttype)
    if "voicemail" == dsttype or "faxes" == dsttype then
        if not extension or "" == extension then
            return nil, "no extension"
        end
        url = url .. string.format("/%s", extension)
    end

    local basename = string.match(file, "^/.+/(.-)$")
    if not basename then
        return nil, "invalid path"
    end
    if ".s3copy" == string.sub(basename, -7) then
        basename = string.sub(basename, 1, -8)
    end
    url = url .. string.format("/%s", basename)
    local cmd = string.format("/usr/bin/sudo %s s3 cp %s %s", script, file, url)

    return 0 == os.execute(cmd)
end

return aws
