--- Authorization backend
-- 
-- @file    authBackend.lua 
-- @author  Vyacheslav Khudan
-- @created 2017/01/17

--- @module authBackend
local authBackend = {}

----------------------------------------------------------------
-- Implementation details
----------------------------------------------------------------
--- Get user data from LDAP
-- @return table with requested exten and admin data
local function __getDataByLdap(exten)
    local rv = {} 
    
    -- Retrieve extension data
    if exten then
        local extenData, errMsg = require"core.wmsldap":get("telephoneNumber", exten)
        if errMsg then
            print(errMsg); logger:error(errMsg)
        end
        if 0 ~= #extenData and extenData[1][2] and extenData[1][2].uid then
            rv[1]    = extenData[1][2]
            rv[1].DN = extenData[1][1]
        end
    end
    
    -- Retrieve admin data
    local adminData, errMsg = require"core.wmsldap":get("cn", "admin")
    if errMsg then
        print(errMsg); logger:error(errMsg)
    end
    if 0 ~= #adminData and adminData[1][2] and adminData[1][2].uid then
        rv[2] = adminData[1][2]
    end
    
    return rv
end

----------------------------------------------------------------
-- Public interface
----------------------------------------------------------------
local backend = {   -- meta class
    source = nil
}   

--- Retrieve user data by extension
function backend:getData(exten)
    local rv = nil
    if "LDAP" == self.source then
        rv = __getDataByLdap(exten)
    end
    
    return rv
end


--- Object constructor 
function authBackend:new(backendSource)
    if "LDAP" ~= backendSource and "AD" ~= backendSource then
        return nil
    end
    
    local obj = {}
    setmetatable(obj, {__index = backend})
    
    obj.source = backendSource
    
    return obj
end

return authBackend

