--- Factory for authorization request environment
-- 
-- @file    authReqEnvFactory.lua 
-- @author  Vyacheslav Khudan
-- @created 2017/01/16

package.path = package.path .. ";/usr/local/share/lua/5.1/common/userAuth/?.lua"

--- @module authReqEnvFactory
local authReqEnvFactory = {
    inputInterfaces = {
         xmlBrowser = "authXmlBrowserRequest"
        ,dialplan   = "authDialplanRequest"
    }
}

----------------------------------------------------------------
-- Public interface
----------------------------------------------------------------
local factory = {  -- meta class
        identArgs = nil
}

--- Create request environment object by ident value
-- @param ident		A identification of the input request
function factory:instantiate(ident)
    if authReqEnvFactory.inputInterfaces[ident] then
        return require(authReqEnvFactory.inputInterfaces[ident]).parse(self.identArgs)
    end
end

--- Create new iterator
-- @param functor	A name of the function which will be executed
function factory:iterator(functor)
    local i = 0
    return function()
        i = i + 1
        if functor and authReqEnvFactory.inputInterfaces[i] then
            local interfaceTag = authReqEnvFactory.inputInterfaces[i]
            local callee = require(authReqEnvFactory.inputInterfaces[interfaceTag])[functor]
            local status = "function" == type(callee) and callee(self.identArgs)
            return interfaceTag, status
        end
    end
end


--- Initialize factory
function authReqEnvFactory:new(args)
    local obj = {}
    setmetatable(obj, {__index = factory})
    obj.identArgs = args
    
    -- add indexing to interfaces table
    for k, v in pairs(self.inputInterfaces) do
        if "string" == type(k) then
            self.inputInterfaces[#self.inputInterfaces + 1] = k
        end
    end
    
    return obj
end

return authReqEnvFactory
