--- A module to processing request from XML browser
-- 
-- @file    authXmlBrowserRequest.lua 
-- @author  Vyacheslav Khudan
-- @created 2017/01/16

package.path = package.path .. ";/usr/local/share/lua/5.1/common/userAuth/?.lua"

--- @module authXmlBrowserRequest
local authXmlBrowserRequest = {}

----------------------------------------------------------------
-- Public interface
----------------------------------------------------------------
--- Request parser
-- @param rawData   The input data from the request  
-- @return parsed data table otherwise nil 
function authXmlBrowserRequest.parse(rawData)
    if "table" ~= type(rawData) then
        return nil
    end
    
    -- parsing table
    local parsingTable = {
         remoteIP       = rawData.remoteIP
        ,userAgent      = rawData.ua
        ,requestorIdent = rawData.source
    }
    
    local data = {}
    for k, v in pairs(parsingTable) do
         data[#data + 1] = k
         data[k]         = v
    end
    
    return 0 ~= #data and data
end

--- Static request checker
-- @param rawData   The input data from the request 
-- @return boolean value 
function authXmlBrowserRequest.staticCheck(rawData)
    local rv = false
    repeat
        if "table" ~= type(rawData) then
            break
        end
        
        if not rawData.ua or not rawData.ua:find("^Wildix WP.*") then
            break
        end
        
        if not rawData.uri or 
         (not rawData.uri:find("action=getxmlbrowser") and
         not rawData.uri:find("action=getphonebook") and
         not rawData.uri:find("action=gethistory")) then
            break
        end
        
        rv = true
    until true
    
    return rv
end

return authXmlBrowserRequest
