--- Defined user features
--
-- @file    defUserFeatures.lua
-- @author  Vyacheslav Khudan
-- @created 2016/05/10

--- @module defUserFeatures
local defUserFeatures = {}

----------------------------------------------------------------
-- Public interface
----------------------------------------------------------------
--- List of the default user statuses
-- @return list of the user status values
function defUserFeatures.statusValues()
    return {
         available = 0
        ,dnd       = 1
        ,away      = 2
    }
end

--- Constructor for default user features
-- @param uid   An user identificator
-- @return user features table
function defUserFeatures.new(uid)
    -- default features list
    local features = {
         _version     = 3
        ,uid          = uid
        ,FLINES       = "0"
        ,FKEYS        = "LINE  ,LINE  "   -- Selected function keys (BLF)
        ,ROSTER       = {}
        ,DEFSTATUS    = {}
        ,CALLCENTER   = {
             callGroups   = {}
            ,enable       = "0"
            ,statusSync   = "1"
        }
        ,CALLCENTER_WHITELIST = {
             enable     = "0"
            ,callGroups = {}
        }
        ,WEBAPP       = {
        	enable 		= "0",
        	url 		= ""
        }
        ,POPUPURL     = {
             url        = ""
            ,incoming   = "click"
            ,outgoing   = "click"
        }
        ,TWOFACTOR    = {
            enable      = "0"
            ,type       = "totp"
            ,app        = ""
            ,secret     = ""
        }
        ,DATETIME     = {
            format    = "d/m/Y"         -- d/m/Y | m/d/Y | d M Y | M d Y | d-m-Y | m-d-Y | Y-m-d | d.m.Y - php like format
            ,is24h    = "1"             -- 1 | 0
        }
        ,AUTOAWAY       = {
            enable      = "0",
            delay       = "10"
        }
        ,WEBPUSH       = {
            enable      = "1"
        }
        ,PHONEBOOK    = ""              -- User selected global phonebooks
        ,CTIOPTIONS   = ""              -- User features like FAX options/headers
        ,CALLBLOCK    = {               -- Flag for blocking user calls
             status     = "0"
        }
        ,SHAREDVM     = {               -- Shared voicemail feature
             status     = "0"
            ,value      = ""
        }
        ,HOTLINE      = {
             status     = "0"
            ,value      = ""
            ,delay      = "0"
        }
        ,CWTONE       = {
            status = "1"
        }
        ,ROAD         = {               -- Ring only active device
            status      = "0"
        }
        ,HIDECID      = {              -- Hide number feature
            status      = "0"
        }
        ,STATUS       = {               -- User status data
             value      = ""
            ,message    = ""
            ,untilTime  = ""
            ,location   = ""
        }
        ,HOTKEY			= {
        			call = ""
        		   ,attendantconsole = ""
        }
        ,ALTERNATE_PICKUP_MODE = {
             callgroups   = ""
            ,pickupgroups = ""
        }
        ,CUSTOM_IDENTITIES = ""
        ,CURRENT_IDENTITY = ""
        ,CALENDAR 		= {
            OUTLOOK = {
                 enable        = "0"
                ,hide_private  = "1"
                ,working_hours = {
                     enable           = "1"
                    ,message          = "Off work"
                    ,status           = "away"
                }
                ,show_message = {
                     Free             = "1"
                    ,WorkingElsewhere = "1"
                    ,Tentative        = "1"
                    ,Busy             = "1"
                    ,OutOfOffice      = "1"
                }
                ,status_map   = {
                     Free             = "available"
                    ,WorkingElsewhere = "available"
                    ,Tentative        = "available"
                    ,Busy             = "dnd"
                    ,OutOfOffice      = "away"
                }
                ,popup = {
                     enable      = "1"
                     ,incoming   = "click"
                     ,outgoing   = "click"
                }
            }
        }
        ,_classes     = {               -- User sub classes with related features (see featuresByStatus)
             internal   = {}
            ,external   = {}
            ,blacklist  = {}
            ,whitelist  = {}
        }
    }

    -- user features specific for every status
    local featuresByStatus = {
        _ACTIVATED      = { status = "0" }
       ,REJECT          = { status = "0" }
       ,BCF             = { status = "1", value = "VOICEMAIL" }
       ,UCF             = { status = "1", value = "VOICEMAIL" }
       ,FCF             = { status = "0", value = "VOICEMAIL" }
       ,CW              = { status = "0" }
       ,TOUT            = { status = "0", value = "" }
       ,MEXT            = { status = "0", value = "10" }
       ,MOBCONFIRMATION = { status = "0" }
       ,SMSMCN          = { status = "0" }
       ,EMAILMCN        = { status = "1" }
       ,RING            = { status = "0", value = "Ring1" }
   }

    -- set up user features classes by user statuses
    local userSubClasses = {"default", "dnd", "away"}
    for _, class in pairs(features._classes) do
        if "table" == type(class) then
            for _, subClass in ipairs(userSubClasses) do
                class[subClass] = require"core.pbxutils".tcopy(featuresByStatus)
            end
        end
    end

    -- update "Internal" class by specific settings
    local internalClass = features._classes.internal
    internalClass["default"]["_ACTIVATED"].status       = "1"
    internalClass["default"]["CW"].status               = "1"
    internalClass["default"]["RING"].status             = "1"

    internalClass["dnd"]["_ACTIVATED"].status           = "1"
    internalClass["dnd"]["REJECT"].status               = "1"
    internalClass["dnd"]["RING"].status                 = "1"
    internalClass["dnd"]["RING"].value                  = "silent"

    internalClass["away"]["CW"].status                  = "1"
    internalClass["away"]["RING"].status                = "1"

    -- update "External" class by specific settings
    local externallClass = features._classes.external
    externallClass["default"]["CW"].status              = "1"
    externallClass["default"]["RING"].status            = "1"
    externallClass["default"]["RING"].value             = "External"

    externallClass["dnd"]["REJECT"].status              = "1"
    externallClass["dnd"]["RING"].status                = "1"
    externallClass["dnd"]["RING"].value                 = "silent"

    externallClass["away"]["CW"].status                 = "1"
    externallClass["away"]["RING"].status               = "1"
    externallClass["away"]["RING"].status               = "1"
    externallClass["away"]["RING"].value                = "External"

    -- update "Blacklist" class by specific settings
    local blacklistClass = features._classes.blacklist
    blacklistClass["default"]["REJECT"].status          = "1"
    blacklistClass["default"]["EMAILMCN"].status        = "0"

    blacklistClass["dnd"]["REJECT"].status              = "1"
    blacklistClass["dnd"]["EMAILMCN"].status            = "0"
    blacklistClass["dnd"]["RING"].value                 = "silent"

    blacklistClass["dnd"]["EMAILMCN"].status            = "0"

    -- update "Whitelist" class by specific settings
    local whitelistClass = features._classes.whitelist
    whitelistClass["default"]["CW"].status              = "1"
    whitelistClass["default"]["MOBCONFIRMATION"].status = "1"

    whitelistClass["dnd"]["REJECT"].status              = "1"
    whitelistClass["dnd"]["MOBCONFIRMATION"].status     = "1"
    whitelistClass["dnd"]["RING"].value                 = "silent"

    whitelistClass["away"]["CW"].status                 = "1"
    whitelistClass["away"]["MOBCONFIRMATION"].status    = "1"

    local countryCode =  require"core.pbxutils".getDialplanGeneralSettings("GCOUNTRY_CODE") or false;
    --Default values for US differ from others
    if countryCode and countryCode == "1" then
        features.DATETIME.format = "m/d/Y"
        features.DATETIME.is24h = "0"
    end

    return features
end

return defUserFeatures

