--- Processing a range of the Wildix devices MAC addresses
-- 
-- @file	wdmacs.lua 
-- @author	Vyacheslav Khudan
-- @created 2015/01/14

local require = require
local tonumber = tonumber
local pairs = pairs
local string = string

module(... or "wdmacs")

----------------------------------------------------------------
-- Implementation details
----------------------------------------------------------------

--- Table with information aboud Wildix devices
-- This table has clone in php code (/GUI/Interface/www/functions.php).
-- It was done in connection with encryption PHP and LUA files.
-- Please don't forget to update these two files !!!
local wdmacs = {
	-- block MACs of old Wildix phones with yealink hw addresses
	{["family"] = "WP",			["start"] = "001565110000", ["end"] = "00156511FFFF", ["UA"] = {["WP400"]=1, ["WP480"]=1, ["WP490"]=1, ["WP500"]=1, ["WP600"]=1}},
	{["family"] = "WP",			["start"] = "001565140000", ["end"] = "00156514FFFF", ["UA"] = {["WP400"]=1, ["WP480"]=1, ["WP490"]=1, ["WP500"]=1, ["WP600"]=1}},
	{["family"] = "WP",			["start"] = "001565160000", ["end"] = "00156516FFFF", ["UA"] = {["WP400"]=1, ["WP480"]=1, ["WP490"]=1, ["WP500"]=1, ["WP600"]=1}},
	{["family"] = "WP",			["start"] = "0015651DAB60", ["end"] = "0015651DAB6F", ["UA"] = {["WP400"]=1, ["WP480"]=1, ["WP490"]=1, ["WP500"]=1, ["WP600"]=1}},

	{["family"] = "WP",			["start"] = "9C7514000000", ["end"] = "9C751401FFFF", ["UA"] = {["WP400"]=1, ["WP480"]=1, ["WP490"]=1, ["WP500"]=1, ["WP600"]=1}},
	{["family"] = "WPR2",		["start"] = "9C7514020000", ["end"] = "9C751402FFFF", ["UA"] = {["WP480R2"]=1}},
	{["family"] = "WPR2",		["start"] = "9C7514040000", ["end"] = "9C751406FFFF", ["UA"] = {["WP480R2"]=1}},
	{["family"] = "WPR2",		["start"] = "9C7514070000", ["end"] = "9C75140DFFFF", ["UA"] = {["WP480R3"]=1, ["WP410R3"]=1}},
	{["family"] = "FXS",		["start"] = "9C7514100000", ["end"] = "9C75141FFFFF", ["UA"] = {["W01FXS"]=1}},
	{["family"] = "FXSR2",		["start"] = "9C751432B200", ["end"] = "9C751432FFFF", ["UA"] = {["W01FXSR2"]=1}},
	{["family"] = "MGW",		["start"] = "9C7514320000", ["end"] = "9C751432B1FF", ["UA"] = {}}, --TODO: fill UA table
	{["family"] = "MGW1",		["start"] = "0090F8000000", ["end"] = "0090F8FFFFFF", ["UA"] = {}}, --TODO: fill UA table
	{["family"] = "FXSR2",		["start"] = "9C7514370000", ["end"] = "9C751437FFFF", ["UA"] = {["W24FXSR2"]=1, ["W24FXSR3"]=1, ["W04FXSR3"]=1}},
	{["family"] = "DWG",		["start"] = "9C7514330000", ["end"] = "9C751433FFFF", ["UA"] = {["W01GSM"]=1, ["DaySaver"]=1}},
	{["family"] = "W-AIR",		["start"] = "9C7514340000", ["end"] = "9C7514342FFF", ["UA"] = {["W-AIR"]=1}},
	{["family"] = "W-AIR SB",	["start"] = "9C7514343000", ["end"] = "9C7514344FFF", ["UA"] = {["W-AIR"]=1}},
    {["family"] = "W-AIR",      ["start"] = "9C7514347000", ["end"] = "9C7514347FFF", ["UA"] = {["W-AIR"]=1}},
    {["family"] = "W-AIR SB",   ["start"] = "9C7514410000", ["end"] = "9C7514411FFF", ["UA"] = {["W-AIR"]=1}}, -- W-AIR SBR2
    {["family"] = "W-AIR",      ["start"] = "00087B000000", ["end"] = "00087BFFFFFF", ["UA"] = {["W-AIR"]=1}}, -- W-AIR outdoor BASE
    {["family"] = "W-PA",       ["start"] = "9C7514381000", ["end"] = "9C7514381FFF", ["UA"] = {["W-PA"]=1}}, -- W-PA
    {["family"] = "W-PA2",      ["start"] = "9C7514382000", ["end"] = "9C7514382FFF", ["UA"] = {["W-PA2"]=1}}, -- W-PA2
    {["family"] = "W-PA2",      ["start"] = "00D84A03ECF0", ["end"] = "00D84A03ECFF", ["UA"] = {["W-PA2"]=1}}, -- W-PA2 sample
    {["family"] = "W-PA",       ["start"] = "0C383E1FBD06", ["end"] = "0C383E1FBD06", ["UA"] = {["W-PA"]=1}}, --PA2 sample
    {["family"] = "W-PA",       ["start"] = "00A834682560", ["end"] = "00A83468256F", ["UA"] = {["W-PA"]=1}}, --PA2 sample bridge mode support
    {["family"] = "FXSR3",      ["start"] = "9C7514310000", ["end"] = "9C7514314FFF", ["UA"] = {["W02FXSR3"]=1}},
    {["family"] = "FXOR3",      ["start"] = "F8A03D200000", ["end"] = "F8A03D30FFFF", ["UA"] = {["W04FXOR3"]=1}}, -- W04FXOR3 (Vendor's sample range of MAC)
    {["family"] = "FXOR3",      ["start"] = "9C7514310000", ["end"] = "9C751431FFFF", ["UA"] = {["W04FXOR3"]=1}}, -- W04FXOR3 (Wildix range of MAC for production)
    {["family"] = "W-AIR",      ["start"] = "9C7514345000", ["end"] = "9C7514346FFF", ["UA"] = {["W-AIR"]=1}}, -- W-AIR SP
    {["family"] = "W-AIR",      ["start"] = "9C7514430000", ["end"] = "9C7514431FFF", ["UA"] = {["W-AIR"]=1}}, -- W-AIR SP indoor and W-AIR SP outdoor BASES
}

local modelAdditionalMacs = {
	 {["model"] = "wp410r2", ["start"] = "9C751402CCCA", ["end"] = "9C751402FFFF", ["version"] = "v2"}
	,{["model"] = "wp410r2", ["start"] = "9C7514040000", ["end"] = "9C75140DFFFF", ["version"] = "v2"}
	,{["model"] = "w04bri",  ["start"] = "0090F80EC000", ["end"] = "0090F8FFFFFF", ["version"] = "C7"}
}

--- Comparing MAC addresses
-- @return 0 -> mac1 = mac2; 1 -> mac1 > mac2; -1 -> mac1 < mac2
local function __maccmp (mac1, mac2)
	if not mac1 or not mac2 then
		return false
	end
	
	local rv = 0
	for i=1,12,2 do
		local mac1_dec = tonumber(mac1:sub(i, i+1), 16)
		local mac2_dec = tonumber(mac2:sub(i, i+1), 16)
	
		if mac1_dec < mac2_dec then
			rv = -1
			break
		elseif mac1_dec > mac2_dec then
			rv = 1
			break
		end
	end
	
	return rv
end

--- Search Wildix device information for specified mac
local function __info_by_mac(mac)
	if not mac then
		return false
	end
	
	local pbxutils = require"core.pbxutils"
	mac = pbxutils.get_mac(mac)
	if not mac then
		return false
	end
	
	local dev_rt = nil
	for _, dev_t in pairs(wdmacs) do
		local end_cmp   = false
		local start_cmp = false
		local end_mac   = nil 
		local start_mac = pbxutils.get_mac(dev_t["start"])
		if start_mac then
			start_cmp = __maccmp(mac, start_mac)
		end
		
		if start_cmp and 0 <= start_cmp then
			end_mac = pbxutils.get_mac(dev_t["end"])
			if end_mac then
				end_cmp = __maccmp(mac, end_mac)
			end
		end
		
		if start_cmp and end_cmp then
			if 0 <= start_cmp and 0 >= end_cmp then
				dev_rt = dev_t
				break
			end
		end
	end
	
	return dev_rt
end

----------------------------------------------------------------
-- Public interface
----------------------------------------------------------------

--- Retreivind device family by MAC address
-- @param mac	A searched device MAC address
-- @return device family otherwise FALSE value
function get_family(mac)
	if not mac then
		return false
	end
	
	local dev_t = __info_by_mac(mac)
	if dev_t and dev_t.family then
		return dev_t.family
	end
	
	return false
end

--- Retreivind device model version by MAC address
-- @param mac	A searched device MAC address
-- @return model version family otherwise FALSE value
function get_version(mac)
	if not mac then
		return false
	end
	local pbxutils = require"core.pbxutils"
	mac = pbxutils.get_mac(mac)
	
	for _, model_t in pairs(modelAdditionalMacs) do
		local end_cmp   = false
		local start_cmp = false
		local end_mac   = nil
		local start_mac = pbxutils.get_mac(model_t["start"])
		if start_mac then
			start_cmp = __maccmp(mac, start_mac)
		end
		
		if start_cmp and 0 <= start_cmp then
			end_mac = pbxutils.get_mac(model_t["end"])
			if end_mac then
				end_cmp = __maccmp(mac, end_mac)
			end
		end
		
		if start_cmp and end_cmp then
			if 0 <= start_cmp and 0 >= end_cmp then
				return model_t["version"]
			end
		end
	end
	
	return false
end

--- Check belonging user agent to specified MAC
-- @param mac	A searched device MAC address
-- @return boolean value TRUE if UA was found otherwise FALSE value
function is_found_agent(mac, ua)
	if not mac or not ua or "" == ua then
		return false
	end
	
	local dev_t = __info_by_mac(mac)
	if dev_t and dev_t["UA"] then
		for agent in pairs(dev_t["UA"]) do
			if agent == ua then
				return true
			end
		end
	end
	
	return false
end
