module( ... or "astman", package.seeall)

--------------------------------------------------------------------------------
-- WARNING: this module must be removed and used new AMIClient                --
--------------------------------------------------------------------------------

local function send (payload, getresponse)
	local getresponse = getresponse or false
	
	-- Reading AMI configuration
    if "table"  ~= type(_M.config) then
        local fp        = io.open("/etc/callweaver/manager.conf")
        local config    = {}
        local content   = fp:read("*a"); fp:close()
        config.password = content:match(".*secret = ([^\n]*).*") or "wildix"
        config.port     = content:match(".*port = ([^\n]*).*") or "5039"
        config.host     = content:match(".*bindaddr = ([^\n]*).*") or "127.0.0.1"
        _M.config       = config
    end
    local password = _M.config.password
    local port     = _M.config.port
    local host     = _M.config.host
	local timeout  = 10
	local sock     = {}

	-- init common functions
	local function init_connection()
		local err
		if nmap then -- set connection via nmap libs
			local socket, status = nmap.new_socket() -- create a socket
			socket:set_timeout(timeout*1000)
			status, err = socket:connect(host, port)
			sock.conn = socket
		else -- set connection via std socket lib
			local socket, conn = require"socket"
			conn, err = socket.connect(host, port)
			if not err then conn:settimeout(timeout) end
			sock.conn = conn
		end
		sock.err = err
		return err
	end
	
	local function close_connection()
		if sock.conn then
			sock.conn:close()
		end
	end
	
	local function receive_data()
		local total
		if nmap then
            local status, result = sock.conn:receive()
            if status then
            	total = result
            else
            	total = ""
            	sock.err = result
            end
            sock.conn:set_timeout(100)
            status, result = sock.conn:receive()
            sock.conn:set_timeout(timeout*1000)
            if status then
            	total = total .. "\n" .. result
            end
            return total
        else
			local result, err = sock.conn:receive()
			if result then total = "" end
            sock.conn:settimeout(0.003)
			while result do
				total = total .. "\n" .. result
				result, err = sock.conn:receive()
			end
			sock.conn:settimeout(timeout)
			if total then err = nil end
			sock.err = err
			return total
		end
	end
	
	local function send_data(data)
		if nmap then
			sock.conn:send(data)
		else
			sock.conn:send(data) 
		end
	end
	
	-- set connection to AMI
	if init_connection() then
		close_connection()
		return false, string.format("Error init connection to AMI: %s", sock.err or "")
	end
	local ans = receive_data()	-- wait for a banner
	
	-- send auth data
	local auth = string.format("Action: Login\r\nUsername: %s\r\nSecret: %s\r\nEvents: off\r\n\r\n", "root", password)
	send_data(auth)
	ans = receive_data()
	if sock.err then
		close_connection()
		return false, string.format("Error action Login to AMI: %s", sock.err or "")
	end
	if ans:match("Response: Error") then
		close_connection()
		return false, string.format("Error AMI: %s", ans:match("Message: (.*)\n"))
	end
	
	-- send payload
	send_data(payload)
	if type(getresponse) == "boolean" then -- simple parsing
		ans = receive_data()
		if sock.err then
			close_connection()
			return false, string.format("Error AMI: %s", sock.err or "")
		end
		return ans
	elseif getresponse and type(getresponse) == "function" then
		sock.close   = close_connection
		sock.send    = send_data
		sock.receive = receive_data
		return getresponse(sock)
	end
	
	close_connection()
	return true
end
	
--- Send Command action
-- @param command A string with callweaver cli command
function send_command (command, getresponse)
   local getresponse = getresponse or false
   local payload = string.format("Action: Command\r\nCommand: %s\r\n\r\n", command)
   return send(payload, getresponse)
end

--- Get value of the global channel
--- @param var A global variable
function get_global_var (var, is_digit)
	if not var then return false end
	
	local payload = string.format("Action: Getvar\r\nVariable: %s\r\n\r\n", var)
	local res, err = send(payload)
	if err or not res then
		return false, err
	end
	
	if res:match("Response%:%s*Success") then
		local result = res:match("Value%:%s(.*)\n")
		if is_digit then
			return tonumber(result)
		else
			return result
		end
	else
		return false
	end	
end

--TODO: ProxyAction, etc.
--- Send Custom Action
-- @param command A string name of action
-- @param data A table with action data
function send_action (action, data, getresponse)
   local getresponse = getresponse or false
   local tdata = {}
   local filter, is_new_exten = "", false
   local tpayload = {}

   -- filter
   if data["filter"] then
	  if data["filter"]["user"] then
		 tpayload[#tpayload+1] = string.format (
			"ProxyAction: ProxySetFilter\r\nOperation: %s\r\nUser: %s\r\nMessage: %s\r\n",	
			data["filter"]["operation"] or "and",
			data["filter"]["user"],
			data["filter"]["message"] or "*"
	     )
	  else
		 tpayload[#tpayload+1] = string.format (
			"ProxyAction: ProxySetFilter\r\nMessage: %s\r\n",
			data["filter"]["message"] or "*"
	     )
	  end
   	  data["filter"] = nil
   end
   for k, v in pairs (data) do
	  tdata[#tdata+1] = string.format ("%s: %s", k, v)
   end

   -- new exten
   if not is_new_exten then
	  tpayload[#tpayload+1] = string.format ("ProxyAction: ProxyNewExtenEvent\r\nEnabled: off\r\n")	--TODO: check this command
   end
   tpayload[#tpayload+1] = string.format("Action: %s\r\n%s\r\n", action, table.concat (tdata, "\r\n"))
   return send (table.concat (tpayload, "\r\n") .. "\r\n", getresponse)
end

--- Executing callweaver DB request (DBDel/DBDelTree/DBPut/DBGet)
function send_dbrequest(method, data)
    local rv = false
    if "table" ~= type(data) or not data.family then
        return rv
    end
    
    local tpayload    = {}
    local getresponse = false 
    if "get" == method then
        if data.key then
            tpayload[#tpayload + 1] = "Action: DBGet"
            tpayload[#tpayload + 1] = "Family: " .. data.family
            tpayload[#tpayload + 1] = "Key: " .. data.key
        end
        getresponse = function(sock)
            local rv   = nil
            local resp = sock.receive()
            if resp:match("DBGetResponse") then
                rv = resp:match("Val:%s([^\n]+)")
            end
            
            sock.close()
            return rv
        end
    elseif "put" == method then
        if data.key and data.val then
            tpayload[#tpayload + 1] = "Action: DBPut"
            tpayload[#tpayload + 1] = "Family: " .. data.family
            tpayload[#tpayload + 1] = "Key: " .. data.key
            tpayload[#tpayload + 1] = "Val: " .. data.val
        end
    elseif "del" == method then
        if data.key then
            tpayload[#tpayload + 1] = "Action: DBDel"
            tpayload[#tpayload + 1] = "Family: " .. data.family
            tpayload[#tpayload + 1] = "Key: " .. data.key
        end
    elseif "deltree" == method then
        tpayload[#tpayload + 1] = "Action: DBDelTree"
        tpayload[#tpayload + 1] = "Family: " .. data.family
    end
    
    local payload = table.concat(tpayload, "\r\n") .. "\r\n\r\n"
    return send(payload, getresponse)
end

--- List currently active channels
-- @param channelIdent   A channel identifier (can be a regular value)
-- @return list with available active channels otherwise empty
function get_active_channels(channelIdent)
    local rv      = {}
    local payload = string.format("Action: CoreShowChannels\r\n\r\n")
    local data    = send(payload)
    if not data or "" == data then
        return rv
    end
    
    if not data:find("Response:%s*Success\n") then
        return rv
    end
    
    if channelIdent then
        channelIdent =  channelIdent:gsub("[%-]", "%%%1")
    end
    
    local listSize    = 0
    local channelData = nil
    for line in data:gmatch("([^\n]+)") do
        -- current channel data processing 
        if line:find("Event:%s*CoreShowChannel") or
           line:find("EventList:%s*Complete") 
        then
            if channelData and channelData.Channel then
                rv[#rv +1] = channelData
            end
            channelData = {}
        end
        
        -- parse channel data items
        if channelData and 
           line:find("^Channel:") or
           line:find("^UniqueID:") or
           line:find("^Context:") or
           line:find("^Extension:") or
           line:find("^BridgedChannel:") or
           line:find("^BridgedUniqueID:") or
           line:find("^ChannelStateDesc:")
        then
            local var, val = line:match("([^:]+):%s*(.+)")
            if var and val and channelData then
                val = "" ~= val and " " ~= val and val
                channelData[var] = val
            end
        end
        
        -- validate channel arument
        if channelIdent and channelData and channelData["Channel"] then
            if not channelData["Channel"]:match(channelIdent) then
                channelData = nil
            end
        end
        
        -- save list size
        if line:find("^ListItems:") then
            listSize = line:match("^ListItems:%s*(%d+)")
            listSize = tonumber(listSize) or 0 
        end
    end
    
    return rv
end
