--- Factory for HTTP request backend
-- 
-- @file    backendFactory.lua 
-- @author  Vyacheslav Khudan
-- @created 2017/11/23

local backendFactory = {}

----------------------------------------------------------------
-- Public interface
----------------------------------------------------------------
local factory = {}  -- meta class

--- Create request backend object
function factory:instantiate(requiredFileLoading, backend)
    if backend == "shell" then
        return require"core.httpClient.callweaverBinCURLBackend"
    elseif backend == "curl" then
        return require"core.httpClient.callweaverCURLBackend"
    elseif backend == "socket" then
        return require"core.httpClient.socketBackend"
    elseif channel and app then
        if requiredFileLoading then
            return require"core.httpClient.callweaverBinCURLBackend"
        end
        return require"core.httpClient.callweaverCURLBackend"
    else
        return require"core.httpClient.socketBackend" 
    end
end


--- Initialize factory
function backendFactory:new()
    local obj = {}
    setmetatable(obj, {__index = factory})
    
    return obj
end

return backendFactory 
