--- SIP proxy response processor
-- 
-- @file    response.lua 
-- @author  Vyacheslav Khudan
-- @created 2017/06/20

local response = {}

----------------------------------------------------------------
-- Internal helpers
----------------------------------------------------------------
local function __isSIPProxyResult(data)
    if "table" ~= type(data) then
        return false
    end
    
    return data.aor and data.useragent
end

----------------------------------------------------------------
-- Result item interface
----------------------------------------------------------------
local item = {}  -- meta class

--- Getter for registered extension
function item:getExten()
    if __isSIPProxyResult(self.data) then
        return self.data.aor
    end
    return nil
end

--- Getter for user agent
function item:getUserAgent()
    if __isSIPProxyResult(self.data) then
        return self.data.useragent or ""
    end
    return ""
end

--- Getter registered address
function item:getAddress()
    if __isSIPProxyResult(self.data) then
        local addressField = self.data.received
        if addressField then
            return addressField:match("sip:([0-9%.]+).*")
        end 
    end
    return nil
end

--- Getter for item data (must be used in debug mode)
function item:getData()
    if __isSIPProxyResult(self.data) then
        return self.data
    end
end


----------------------------------------------------------------
-- Wrapper public interface
----------------------------------------------------------------
local wrapper = {}  -- meta class

-- Iterator over object data
function wrapper:iterate()
    local idx = 0
    return function()
        idx = idx + 1
        if "table" == type(self.list[idx]) then
            return setmetatable({data = self.list[idx]}, {__index = item})
        end
    end
end

--- Retrieve first item from result
function wrapper:first()
    return setmetatable({data = self.list[1]}, {__index = item})
end


-- Wrapper builder
function response.wrap(result)
    if "table" == type(result) then
        return setmetatable({list = result}, {__index = wrapper})
    end
    return result
end

return response